@@ -1,6 +1,6 @@
 # This file is part of the Ubuntu Continuous Integration test tools
 #
-# Copyright 2014 Canonical Ltd.
+# Copyright 2014, 2015 Canonical Ltd.
 #
 # This program is free software: you can redistribute it and/or modify it under
 # the terms of the GNU General Public License version 3, as published by the
@@ -45,6 +45,9 @@ class Feature(object):
     def feature_name(self):
         return self.__class__.__name__
 
+    def __repr__(self):
+        return self.feature_name()
+
     def __unicode__(self):
         return self.feature_name()
 
@@ -76,8 +79,8 @@ class PlatformFeature(Feature):
 
     def __init__(self):
         super(PlatformFeature, self).__init__()
-        # self.distro strcture may vary between distros, let the daughter
-        # classes handle the differences.
+        # self.distro structure may vary between distros, the daughter classes
+        # handle the differences.
         self.distro = platform.linux_distribution()
 
 
@@ -97,17 +100,27 @@ class UbuntuPlatformFeature(PlatformFeature):
 
     @property
     def version(self):
-        """The Ubuntu version (yy.mm) string."""
-        # This is a property, but it should be available even when we run on a
-        # different version, so we *don't* call _probe()
+        """The Ubuntu version (yy.mm) string.
+
+        This property is available even when we run on a different version,
+        so we *don't* call _probe().
+        """
         return self._version
 
 
 trusty = UbuntuPlatformFeature('trusty')
+utopic = UbuntuPlatformFeature('utopic')
+vivid = UbuntuPlatformFeature('vivid')
+wily = UbuntuPlatformFeature('wily')
+xenial = UbuntuPlatformFeature('xenial')
 
 
 def requires(feature):
-    """A decorator to skip a test if a feature is not available."""
+    """A decorator to skip a test if a feature is not available.
+
+    If the feature is not available, a test case or a test suite using this
+    decorator will be skipped.
+    """
     if not feature.available():
         reason = '{} is not available'.format(feature.feature_name())
         return unittest.skip(reason)
