@@ -1,6 +1,6 @@
 # This file is part of the Ubuntu Continuous Integration test tools
 #
-# Copyright 2013, 2014 Canonical Ltd.
+# Copyright 2013-2016 Canonical Ltd.
 #
 # This program is free software: you can redistribute it and/or modify it under
 # the terms of the GNU General Public License version 3, as published by the
@@ -19,6 +19,9 @@ import re
 import unittest
 
 
+import testtools
+
+
 def filter_suite(condition, suite):
     """Return tests for which ``condition`` is True in ``suite``.
 
@@ -26,7 +29,7 @@ def filter_suite(condition, suite):
         test should be kept.
 
     :param suite: A test suite that can be iterated. It contains either tests
-        or suite inheriting from ``unittest.TestSuite``.
+        or a suite inheriting from ``unittest.TestSuite``.
 
     ``suite`` is a tree of tests and suites, the returned suite respect the
     received suite layout, only removing empty suites.
@@ -45,6 +48,38 @@ def filter_suite(condition, suite):
     return filtered_suite
 
 
+def pick(suite, rank, modulo):
+    """Pick some tests in ``suite```.
+
+    :param suite: The suite to pick tests from.
+
+    :param rank: The test to pick every ``modulo`` tests when iterating
+        ``suite``.
+
+    :param modulo: How often is ``rank`` picked.
+
+    :returns: A suite of tests picking the ``rank``nth test every ``modulo``
+        tests from ``suite``.
+
+    """
+    if not modulo:
+        # modulo == zero is a special case, we keep all tests
+        return suite
+    # global but declared inside the func so the inner 'nth' func can modify it
+    global __nth_calls
+    __nth_calls = 0
+
+    def nth(test):
+        global __nth_calls
+        try:
+            if __nth_calls + 1 == rank:
+                return True
+        finally:
+            __nth_calls = (__nth_calls + 1) % modulo
+        return False
+    return filter_suite(nth, suite)
+
+
 def partition_suite(suite, count):
     """Partition suite into ``count`` suites.
 
@@ -55,11 +90,15 @@ def partition_suite(suite, count):
     :returns: A list of ``count`` suites respecting the ``suite`` layout,
         dropping inner empty suites.
     """
+    # testtools.ConcurrentTestSuite wraps tests in a list which prohibits
+    # filter_suite to preserve the original suites used, unwrap it
+    if isinstance(suite, testtools.ConcurrentTestSuite):
+        suite = suite._tests[0]
     # This just assigns tests in a round-robin fashion.  On one hand this
-    # splits up blocks of related tests that might run faster if they shared
+    # splits up blocks of related tests that might run faster if they share
     # resources, but on the other it avoids assigning blocks of slow tests to
     # just one partition.  So the slowest partition shouldn't be much slower
-    # than the fastest.
+    # than the fastest (confirmed IRL, not a metric, etc ;).
     global __nth_calls  # global but declared inside the func
     suites = []
     for cur in range(count):
@@ -145,3 +184,54 @@ def exclude_regexps(regexps, suite):
                 return False
         return True
     return filter_suite(matches_none_of, suite)
+
+
+def load_test_id_list(file_handle):
+    """Load a test id list from a file handle.
+
+    The format is one test id by line.  No special care is taken to impose
+    strict rules, these test ids are used to filter the test suite so a test id
+    that do not match an existing test will do no harm. This allows user to add
+    comments, leave blank lines, etc as they won't match existing test ids.
+
+    On the flip side, this means no control can be made about typos (a
+    non-matching test id won't be run) but in practice, the test ids are copied
+    from a previous run so typos are unlikely.
+    """
+    test_list = []
+    for test_name in file_handle.readlines():
+        test_list.append(test_name.strip())
+    return test_list
+
+
+class TestIdList(object):
+    """Test id list to filter a test suite.
+
+    Relying on the assumption that test ids are built as:
+    <module>[.<class>.<method>][(<param>+)], <module> being in python dotted
+    notation, this class offers methods to :
+    - keep only the listed tests from a given suite.
+    """
+
+    def __init__(self, test_id_list):
+        # When a test suite needs to be filtered against us we compare test ids
+        # for strict equality, so a simple dict offers a quick and simple
+        # solution.
+        self.tests = dict().fromkeys(test_id_list, True)
+
+    def includes(self, test_id):
+        return test_id in self.tests
+
+
+def include_id_list(id_list, suite):
+    """Returns the tests whose id are part of the list.
+
+    :param id_list: A TestIdList describing the tests that should be kept.
+
+    :param suite: The test suite to filter.
+    """
+    if not id_list:
+        # No id list, no filtering
+        return suite
+
+    return filter_suite(lambda t: id_list.includes(t.id()), suite)
