@@ -1,6 +1,6 @@
 # This file is part of the Ubuntu Continuous Integration test tools
 #
-# Copyright 2013, 2014 Canonical Ltd.
+# Copyright 2013, 2014, 2015 Canonical Ltd.
 #
 # This program is free software: you can redistribute it and/or modify it under
 # the terms of the GNU General Public License version 3, as published by the
@@ -50,6 +50,7 @@ import sys
 import traceback
 import unittest
 
+
 from ucitests import matchers
 
 
@@ -83,7 +84,7 @@ class Loader(unittest.TestLoader):
         :param root: An alternate starting point for the loading. Defaults
             to current directory.
         """
-        root = kwargs.pop('root', os.getcwdu())
+        root = kwargs.pop('root', os.getcwd())
         file_matcher = kwargs.pop(
             'file_matcher', matchers.NameMatcher(includes=[r'^test.*\.py$']))
         # MISSINGTEST: The default dir matcher should be restricted to valid
