@@ -1,6 +1,6 @@
 # This file is part of the Ubuntu Continuous Integration test tools
 #
-# Copyright 2013, 2014 Canonical Ltd.
+# Copyright 2013, 2014, 2015 Canonical Ltd.
 #
 # This program is free software: you can redistribute it and/or modify it under
 # the terms of the GNU General Public License version 3, as published by the
@@ -15,8 +15,10 @@
 # this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from __future__ import unicode_literals
+import sys
 import testtools
 from testtools import testresult
+import unittest
 
 
 class TextResult(testresult.TextTestResult):
@@ -31,37 +33,65 @@ class TextResult(testresult.TextTestResult):
             # -- vila 2014-01-28
             super(TextResult, self).__init__(stream)
         self.verbose = verbosity > 1
+        # Output is delayed until 'stopTest' is called. This ensures the output
+        # is produced from a single place at a precise point in time (they are
+        # cases where addError is called multiple times and delaying the output
+        # provides a more consistent behavior).
+        self.delayed_output = ''
 
     def startTest(self, test):
         if self.verbose:
-            self.stream.write(test.id())
+            if sys.version_info < (3,):
+                test_id = test.id().decode('utf8')
+            else:
+                test_id = test.id()
+            self.stream.write(test_id)
             self.stream.write(' ... ')
             self.stream.flush()
         self.start_time = self._now()
         super(TextResult, self).startTest(test)
 
+    def convert_delta_to_float(self, a_timedelta):
+        return (a_timedelta.days * 86400.0 + a_timedelta.seconds +
+                a_timedelta.microseconds / 1000000.0)
+
     def stopTest(self, test):
+        self.stream.write(self.delayed_output)
+        self.delayed_output = ''
         if self.verbose:
             elapsed_time = self._now() - self.start_time
             self.stream.write(' (%.3f secs)\n'
-                              % self._delta_to_float(elapsed_time))
-            self.stream.flush()
+                              % self.convert_delta_to_float(elapsed_time))
+        self.stream.flush()
         super(TextResult, self).stopTest(test)
 
+    def startTestRun(self):
+        super(TextResult, self).startTestRun()
+        # Activate signal handling so KeyboardInterrupt use the same internal
+        # plumbing as failfast
+        unittest.installHandler()
+        unittest.registerResult(self)
+
+    def stopTestRun(self):
+        super(TextResult, self).stopTestRun()
+        for reason, skipped in self.skip_reasons.items():
+            if not reason:
+                reason = 'No reason given'
+            self.stream.write('{} skipped: {}\n'.format(
+                len(skipped), reason))
+
     def addError(self, test, err=None, details=None):
         if self.verbose:
-            self.stream.write('ERROR')
+            self.delayed_output += 'ERROR'
         else:
-            self.stream.write('E')
-        self.stream.flush()
+            self.delayed_output += 'E'
         super(TextResult, self).addError(test, err, details)
 
     def addFailure(self, test, err=None, details=None):
         if self.verbose:
-            self.stream.write('FAIL')
+            self.delayed_output += 'FAIL'
         else:
-            self.stream.write('F')
-        self.stream.flush()
+            self.delayed_output += 'F'
         super(TextResult, self).addFailure(test, err, details)
 
     def addSkip(self, test, reason):
@@ -70,32 +100,28 @@ class TextResult(testresult.TextTestResult):
                 reason_displayed = ''
             else:
                 reason_displayed = ' ' + reason
-            self.stream.write('SKIP%s' % reason_displayed)
+            self.delayed_output += 'SKIP{}'.format(reason_displayed)
         else:
-            self.stream.write('s')
-        self.stream.flush()
+            self.delayed_output += 's'
         super(TextResult, self).addSkip(test, reason)
 
     def addSuccess(self, test, details=None):
         if self.verbose:
-            self.stream.write('OK')
+            self.delayed_output += 'OK'
         else:
-            self.stream.write('.')
-        self.stream.flush()
+            self.delayed_output += '.'
         super(TextResult, self).addSuccess(test, details)
 
     def addExpectedFailure(self, test, err=None):
         if self.verbose:
-            self.stream.write('XFAIL')
+            self.delayed_output += 'XFAIL'
         else:
-            self.stream.write('x')
-        self.stream.flush()
+            self.delayed_output += 'x'
         super(TextResult, self).addExpectedFailure(test, err)
 
     def addUnexpectedSuccess(self, test):
         if self.verbose:
-            self.stream.write('NOTOK')
+            self.delayed_output += 'NOTOK'
         else:
-            self.stream.write('u')
-        self.stream.flush()
+            self.delayed_output += 'u'
         super(TextResult, self).addUnexpectedSuccess(test)
