@@ -1,6 +1,6 @@
 # This file is part of the Ubuntu Continuous Integration test tools
 #
-# Copyright 2013, 2014 Canonical Ltd.
+# Copyright 2013, 2014, 2015 Canonical Ltd.
 #
 # This program is free software: you can redistribute it and/or modify it under
 # the terms of the GNU General Public License version 3, as published by the
@@ -95,6 +95,18 @@ class TestCwdToTmp(unittest.TestCase):
         # The current directory has been restored
         self.assertEqual(self.before, os.getcwd())
 
+    def test_dir_default_prefix(self):
+        fixtures.set_uniq_cwd(self)
+        cur = os.getcwd()
+        # Default is to name the tmp dir after the test method
+        prefix = self._testMethodName
+        self.assertTrue(os.path.basename(cur).startswith(prefix))
+
+    def test_dir_prefix_provided(self):
+        fixtures.set_uniq_cwd(self, 'provided')
+        cur = os.getcwd()
+        self.assertTrue(os.path.basename(cur).startswith('provided'))
+
 
 class TestProtectImports(unittest.TestCase):
 
