@@ -1,6 +1,6 @@
 # This file is part of the Ubuntu Continuous Integration test tools
 #
-# Copyright 2013, 2014 Canonical Ltd.
+# Copyright 2013, 2014, 2015 Canonical Ltd.
 #
 # This program is free software: you can redistribute it and/or modify it under
 # the terms of the GNU General Public License version 3, as published by the
@@ -18,7 +18,6 @@ import os
 import unittest
 import sys
 
-
 from ucitests import (
     assertions,
     filters,
@@ -28,6 +27,10 @@ from ucitests import (
 )
 
 
+if sys.version_info >= (3,):
+    unicode = str
+
+
 class TestImportFromPath(unittest.TestCase):
 
     def setUp(self):
@@ -40,8 +43,13 @@ class TestImportFromPath(unittest.TestCase):
             f.write('')
         with self.assertRaises(TypeError) as cm:
             self.loader.importFromPath('.')
-        self.assertEqual("relative imports require the 'package' argument",
-                         unicode(cm.exception))
+        if sys.version_info < (3,):
+            self.assertEqual("relative imports require the 'package' argument",
+                             unicode(cm.exception))
+        else:
+            self.assertEqual(
+                "the 'package' argument is required to perform a relative "
+                "import for '.'", str(cm.exception))
 
     def test_weird_path(self):
         d = './foo.bar/baz'
@@ -50,7 +58,8 @@ class TestImportFromPath(unittest.TestCase):
             f.write('')
         with self.assertRaises(ImportError) as cm:
             self.loader.importFromPath(d)
-        assertions.assertMultiLineAlmostEqual(self, '''\
+        if sys.version_info < (3,):
+            assertions.assertMultiLineAlmostEqual(self, '''\
 Failed to import foo.bar.baz at ./foo.bar/baz:
 Traceback (most recent call last):
   File "...", line ..., in importFromPath
@@ -59,7 +68,29 @@ Traceback (most recent call last):
     __import__(name)
 ImportError: No module named foo.bar.baz
 ''',
-                                              unicode(cm.exception))
+                                                  unicode(cm.exception))
+        else:
+            assertions.assertMultiLineAlmostEqual(self, '''\
+Failed to import foo.bar.baz at ./foo.bar/baz:
+Traceback (most recent call last):
+  File "...", line ..., in importFromPath
+    return importlib.import_module(mod_name)
+  File "...", line ..., in import_module
+  ...
+  ...
+  ...
+  ...
+  ...
+  ...
+  ...
+  ...
+  ...
+  ...
+  ...
+  ...
+ImportError: No module named 'foo'
+''',
+                                                  str(cm.exception))
 
     def test_invalid_python_file(self):
         with open('foo.py', 'w') as f:
@@ -72,7 +103,8 @@ ImportError: No module named foo.bar.baz
             f.write("I'm no python code and I'm not even pretending")
         with self.assertRaises(ImportError) as cm:
             self.loader.importFromPath('./foo')
-        assertions.assertMultiLineAlmostEqual(self, '''\
+        if sys.version_info < (3,):
+            assertions.assertMultiLineAlmostEqual(self, '''\
 Failed to import foo at ./foo:
 Traceback (most recent call last):
   File "...", line ..., in importFromPath
@@ -81,12 +113,27 @@ Traceback (most recent call last):
     __import__(name)
 ImportError: No module named foo
 ''',
-                                              unicode(cm.exception))
+                                                  unicode(cm.exception))
+        else:
+            assertions.assertMultiLineAlmostEqual(self, '''\
+Failed to import foo at ./foo:
+Traceback (most recent call last):
+  File "...", line ..., in importFromPath
+    return importlib.import_module(mod_name)
+  File "...", line ..., in import_module
+  ...
+  ...
+  ...
+  ...
+ImportError: No module named 'foo'
+''',
+                                                  unicode(cm.exception))
 
     def test_unknown_file(self):
         with self.assertRaises(ImportError) as cm:
             self.loader.importFromPath('./foo.py')
-        assertions.assertMultiLineAlmostEqual(self, '''\
+        if sys.version_info < (3,):
+            assertions.assertMultiLineAlmostEqual(self, '''\
 Failed to import foo at ./foo.py:
 Traceback (most recent call last):
   File "...", line ..., in importFromPath
@@ -95,7 +142,21 @@ Traceback (most recent call last):
     __import__(name)
 ImportError: No module named foo
 ''',
-                                              unicode(cm.exception))
+                                                  unicode(cm.exception))
+        else:
+            assertions.assertMultiLineAlmostEqual(self, '''\
+Failed to import foo at ./foo.py:
+Traceback (most recent call last):
+  File "...", line ..., in importFromPath
+    return importlib.import_module(mod_name)
+  File "...", line ..., in import_module
+  ...
+  ...
+  ...
+  ...
+ImportError: No module named 'foo'
+''',
+                                                  unicode(cm.exception))
 
     def test_valid_file(self):
         self.assertIs(None, sys.modules.get('foo', None))
@@ -142,7 +203,10 @@ imported = True
         with self.assertRaises(ImportError) as cm:
             sys_path_entry, rel_path = self.loader.packageSysPathFromName(
                 'foo')
-        self.assertEqual('No module named foo', unicode(cm.exception))
+        if sys.version_info < (3,):
+            self.assertEqual('No module named foo', unicode(cm.exception))
+        else:
+            self.assertEqual("No module named 'foo'", str(cm.exception))
 
 
 class TestSortNames(unittest.TestCase):
@@ -226,12 +290,12 @@ class TestIterFlat(unittest.TestCase):
 class TestAssertTestIDs(unittest.TestCase):
 
     def test_single(self):
-        assertTestIDs(self, ['ucitests.fixtures.Test.test_pass'],
+        assertTestIDs(self, ['ucitests.fixtures.UnittestTest.test_pass'],
                       fixtures.make_case())
 
     def test_several(self):
-        assertTestIDs(self, ['ucitests.fixtures.Test.test_pass',
-                             'ucitests.fixtures.Test.test_pass'],
+        assertTestIDs(self, ['ucitests.fixtures.UnittestTest.test_pass',
+                             'ucitests.fixtures.UnittestTest.test_pass'],
                       [fixtures.make_case(), fixtures.make_case()])
 
 
@@ -730,7 +794,7 @@ import unittest
 class T(unittest.TestCase):
   def test_pass(self): pass
 ''')
-        root = os.path.join(os.getcwdu(), 't')
+        root = os.path.join(os.getcwd(), 't')
         sys.path.insert(0, root)
         subloader = self.loader.SubLoader(root=root)
         assertTestIDs(self, ['t1.test.T.test_pass'],
@@ -752,7 +816,7 @@ class T(unittest.TestCase):
   def test_pass(self): pass
 ''')
         sys.path.pop(0)  # Remove the current directory
-        sys.path.insert(0, os.path.join(os.getcwdu(), 't'))
+        sys.path.insert(0, os.path.join(os.getcwd(), 't'))
         assertTestIDs(self, ['t1.test.T.test_pass'],
                       self.loader.loadTestsFromSysPathModule('t1'))
         assertTestIDs(self, ['t2.test.T.test_pass'],
