@@ -1,6 +1,6 @@
 # This file is part of the Ubuntu Continuous Integration test tools
 #
-# Copyright 2013, 2014 Canonical Ltd.
+# Copyright 2013-2016 Canonical Ltd.
 #
 # This program is free software: you can redistribute it and/or modify it under
 # the terms of the GNU General Public License version 3, as published by the
@@ -14,8 +14,9 @@
 # You should have received a copy of the GNU General Public License along with
 # this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from cStringIO import StringIO
 import io
+import os
+import sys
 import unittest
 
 
@@ -35,14 +36,14 @@ class TestTextResultOutput(unittest.TestCase):
 
     def assertOutput(self, expected, kind):
         test = fixtures.make_case(kind)
-        out = StringIO()
+        out = io.StringIO()
         res = results.TextResult(out)
 
         # We don't care about timing here so we always return 0 which
         # simplifies matching the expected result
         def zero(atime):
             return 0.0
-        res._delta_to_float = zero
+        res.convert_delta_to_float = zero
         test.run(res)
         self.assertEqual(expected, res.stream.getvalue())
 
@@ -118,14 +119,14 @@ class TestVerboseResultOutput(unittest.TestCase):
     def assertOutput(self, template, kind):
         test = fixtures.make_case(kind)
         expected = expand_template_for_test(template, test)
-        out = StringIO()
+        out = io.StringIO()
         res = results.TextResult(out, verbosity=2)
 
         # We don't care about timing here so we always return 0 which
         # simplifies matching the expected result
         def zero(atime):
             return 0.0
-        res._delta_to_float = zero
+        res.convert_delta_to_float = zero
         test.run(res)
         self.assertEqual(expected, res.stream.getvalue())
 
@@ -155,7 +156,7 @@ class TestVerboseResultOutput(unittest.TestCase):
 
     def test_skip_reason(self):
         self.assertOutput('''\
-{classname}.{name} ... SKIP Because (0.000 secs)
+{classname}.{name} ... SKIP Reason (0.000 secs)
 ''',
                           'skip_reason')
 
@@ -172,12 +173,8 @@ class TestVerboseResultOutput(unittest.TestCase):
                           'unexpected_success')
 
 
-def run_with_subunit(test):
+def run_with_subunit(test, stream):
         """Run a suite returning the subunit stream."""
-        if tests.minimal_testtools.available():
-            stream = io.BytesIO()
-        else:
-            stream = StringIO()
         res = subunit.TestProtocolClient(stream)
         test.run(res)
         return res, stream
@@ -196,12 +193,17 @@ def assertSubunitOutput(test, template, kind, kwargs=None):
     """
     if kwargs is None:
         kwargs = dict()
-    test_to_run = test.make_case(kind)
-    res, stream = run_with_subunit(test_to_run)
-    assertions.assertMultiLineAlmostEqual(
-        test,
-        expand_template_for_test(template, test_to_run, kwargs),
-        stream.getvalue().decode('utf-8'))
+    fixtures.set_uniq_cwd(test)
+    test.addCleanup(os.remove, 'subunit.stream')
+    with io.open('subunit.stream', 'wb') as stream:
+        test_to_run = test.make_case(kind)
+        res = run_with_subunit(test_to_run, stream)
+    with io.open('subunit.stream', 'rb') as stream:
+        assertions.assertMultiLineAlmostEqual(
+            test,
+            expand_template_for_test(template, test_to_run, kwargs),
+            stream.read().decode('utf-8'))
+    return res
 
 
 class TestSubunitOutputForUnittest(unittest.TestCase):
@@ -230,7 +232,8 @@ AssertionError
                             'fail')
 
     def test_error(self):
-        assertSubunitOutput(self, '''\
+        if sys.version_info < (3,):
+            assertSubunitOutput(self, '''\
 test: {classname}.{name}
 error: {classname}.{name} [
 Traceback (most recent call last):
@@ -242,7 +245,21 @@ Traceback (most recent call last):
 SyntaxError: invalid syntax
 ]
 ''',
-                            'error')
+                                'error')
+        else:
+            assertSubunitOutput(self, '''\
+test: {classname}.{name}
+error: {classname}.{name} [
+Traceback (most recent call last):
+  File "{traceback_file}", line {traceback_line}, in {name}
+    (self.traceback_file, 1, 1, 'No python'))
+  File "{traceback_file}", line 1
+    No python
+    ^
+SyntaxError: invalid syntax
+]
+''',
+                                'error')
 
     def test_skip(self):
         assertSubunitOutput(self, '''\
@@ -257,16 +274,17 @@ skip: {classname}.{name} [
         assertSubunitOutput(self, '''\
 test: {classname}.{name}
 skip: {classname}.{name} [
-Because
+Reason
 ]
 ''',
                             'skip_reason')
 
     @features.requires(tests.minimal_testtools)
     def test_expected_failure(self):
-        assertSubunitOutput(self, '''\
-test: ucitests.fixtures.Test.test_expected_failure
-xfail: ucitests.fixtures.Test.test_expected_failure [
+        if sys.version_info < (3,):
+            assertSubunitOutput(self, '''\
+test: ucitests.fixtures.UnittestTest.test_expected_failure
+xfail: ucitests.fixtures.UnittestTest.test_expected_failure [
 Traceback (most recent call last):
   File "{traceback_file}", line {traceback_line}, in {name}
     self.assertEqual(1, 0, "1 should be 0")
@@ -277,7 +295,22 @@ Traceback (most recent call last):
 AssertionError: 1 should be 0
 ]
 ''',
-                            'expected_failure')
+                                'expected_failure')
+        else:
+            assertSubunitOutput(self, '''\
+test: ucitests.fixtures.UnittestTest.test_expected_failure
+xfail: ucitests.fixtures.UnittestTest.test_expected_failure [
+Traceback (most recent call last):
+  File "{traceback_file}", line {traceback_line}, in {name}
+    self.assertEqual(1, 0, "1 should be 0")
+  File ..., in assertEqual
+    assertion_func(first, second, msg=msg)
+  File ..., in _baseAssertEqual
+    raise self.failureException(msg)
+AssertionError: 1 != 0 : 1 should be 0
+]
+''',
+                                'expected_failure')
 
     def test_unexpected_success(self):
         assertSubunitOutput(self, '''\
@@ -319,7 +352,8 @@ AssertionError
                             dict(traceback_fixed_length=106))
 
     def test_error(self):
-        assertSubunitOutput(self, '''\
+        if sys.version_info < (3,) and not features.xenial.available():
+            assertSubunitOutput(self, '''\
 test: {classname}.{name}
 error: {classname}.{name} [ multipart
 Content-Type: text/x-traceback;...language=python...
@@ -332,8 +366,25 @@ SyntaxError: None
 0\r
 ]
 ''',
-                            'error',
-                            dict(traceback_fixed_length=100))
+                                'error',
+                                dict(traceback_fixed_length=100))
+        else:
+            assertSubunitOutput(self, '''\
+test: {classname}.{name}
+error: {classname}.{name} [ multipart
+Content-Type: text/x-traceback;...language=python...
+traceback
+{subunit_traceback_length}\r
+Traceback (most recent call last):
+  File "{traceback_file}", line {traceback_line}, in {name}
+    raise SyntaxError
+  File "<string>", line None
+SyntaxError: <no detail available>
+0\r
+]
+''',
+                                'error',
+                                dict(traceback_fixed_length=146))
 
     @features.requires(tests.minimal_testtools)
     def test_skip(self):
@@ -354,8 +405,8 @@ test: {classname}.{name}
 skip: {classname}.{name} [ multipart
 Content-Type: text/plain;charset=utf8
 reason
-7\r
-Because0\r
+6\r
+Reason0\r
 ]
 ''',
                             'skip_reason')
@@ -386,7 +437,8 @@ MismatchError: 1 != 0
     @features.requires(tests.at_least_trusty)
     @features.requires(tests.minimal_testtools)
     def test_expected_failure(self):
-        assertSubunitOutput(self, '''\
+        if sys.version_info < (3,) and not features.xenial.available():
+            assertSubunitOutput(self, '''\
 test: {classname}.{name}
 xfail: {classname}.{name} [ multipart
 Content-Type: text/plain;charset=utf8
@@ -401,7 +453,24 @@ MismatchError: 1 != 0
 0\r
 ]
 ''',
-                            'expected_failure')
+                                'expected_failure')
+        else:
+            assertSubunitOutput(self, '''\
+test: {classname}.{name}
+xfail: {classname}.{name} [ multipart
+Content-Type: text/plain;charset=utf8
+reason
+D\r
+1 should be 00\r
+Content-Type: text/x-traceback;...language=python...
+traceback
+52\r
+Traceback (most recent call last):
+...MismatchError: 1 != 0
+0\r
+]
+''',
+                                'expected_failure')
 
     @features.requires(tests.minimal_testtools)
     def test_unexpected_success(self):
@@ -441,6 +510,6 @@ class TestSubunitInputStreamTextResultOutput(TestTextResultOutput):
         # simplifies matching the expected result
         def zero(atime):
             return 0.0
-        text_result._delta_to_float = zero
+        text_result.convert_delta_to_float = zero
         receiver.run(text_result)
         self.assertEqual(expected, out.getvalue())
