@@ -1,6 +1,6 @@
 # This file is part of the Ubuntu Continuous Integration test tools
 #
-# Copyright 2014 Canonical Ltd.
+# Copyright 2014, 2015 Canonical Ltd.
 #
 # This program is free software: you can redistribute it and/or modify it under
 # the terms of the GNU General Public License version 3, as published by the
@@ -15,6 +15,7 @@
 # this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import os
+import sys
 import unittest
 
 
@@ -24,6 +25,10 @@ from ucitests import (
 )
 
 
+if sys.version_info >= (3,):
+    unicode = str
+
+
 class TestPathFromRoot(unittest.TestCase):
 
     def test_just_root(self):
